@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: LGPL-2.1+
 /* NetworkManager Applet -- allow user control over networking
  *
- * Copyright (C) 2011-2017 Red Hat, Inc.
+ * Copyright (C) 2011-2022 Red Hat, Inc.
  */
 
 #ifndef NMA_VERSION_H
@@ -31,7 +31,7 @@
  * Evaluates to the micro version number of NetworkManager which this source
  * compiled against.
  */
-#define NMA_MICRO_VERSION (32)
+#define NMA_MICRO_VERSION (34)
 
 /**
  * NMA_CHECK_VERSION:
@@ -61,6 +61,7 @@
 #define NMA_VERSION_1_8_22 (NMA_ENCODE_VERSION (1, 8, 22))
 #define NMA_VERSION_1_8_26 (NMA_ENCODE_VERSION (1, 8, 26))
 #define NMA_VERSION_1_8_28 (NMA_ENCODE_VERSION (1, 8, 28))
+#define NMA_VERSION_1_8_34 (NMA_ENCODE_VERSION (1, 8, 34))
 
 /* For releases, NMA_API_VERSION is equal to NMA_VERSION.
  *
@@ -143,4 +144,10 @@
 # define NMA_AVAILABLE_IN_1_8_28
 #endif
 
+#if NMA_VERSION_MAX_ALLOWED < NMA_VERSION_1_8_34
+# define NMA_AVAILABLE_IN_1_8_34            G_UNAVAILABLE(1.8,34)
+#else
+# define NMA_AVAILABLE_IN_1_8_34
+#endif
+
 #endif  /* NMA_VERSION_H */
