@@ -2,7 +2,7 @@
 /*
  * Dan Williams <dcbw@redhat.com>
  *
- * Copyright 2007 - 2019 Red Hat, Inc.
+ * Copyright (C) 2007 - 2021 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -28,6 +28,7 @@ struct _NMAEapPeap {
 	NMAWs8021x *ws_8021x;
 	gboolean is_editor;
 	GtkWidget *ca_cert_chooser;
+	GtkWidget *eap_widget;
 };
 
 static void
@@ -71,7 +72,7 @@ ca_cert_not_required_toggled (GtkWidget *button, gpointer user_data)
 	NMAEapPeap *method = (NMAEapPeap *) user_data;
 
 	gtk_widget_set_sensitive (method->ca_cert_chooser,
-	                          !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+	                          !gtk_check_button_get_active (GTK_CHECK_BUTTON (button)));
 }
 
 static void
@@ -205,33 +206,30 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 {
 	NMAEap *parent = (NMAEap *) user_data;
 	NMAEapPeap *method = (NMAEapPeap *) parent;
-	GtkWidget *vbox;
+	GtkBox *vbox;
 	NMAEap *eap = NULL;
-	GList *elt, *children;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GtkWidget *eap_widget;
 
-	vbox = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_inner_auth_vbox"));
-	g_assert (vbox);
+	vbox = GTK_BOX (gtk_builder_get_object (parent->builder, "eap_peap_inner_auth_vbox"));
+	g_return_if_fail (vbox);
 
 	/* Remove any previous wireless security widgets */
-	children = gtk_container_get_children (GTK_CONTAINER (vbox));
-	for (elt = children; elt; elt = g_list_next (elt))
-		gtk_container_remove (GTK_CONTAINER (vbox), GTK_WIDGET (elt->data));
+	if (method->eap_widget)
+		gtk_box_remove (vbox, method->eap_widget);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter);
 	gtk_tree_model_get (model, &iter, I_METHOD_COLUMN, &eap, -1);
 	g_assert (eap);
 
-	eap_widget = nma_eap_get_widget (eap);
-	g_assert (eap_widget);
-	gtk_widget_unparent (eap_widget);
+	method->eap_widget = nma_eap_get_widget (eap);
+	g_return_if_fail (method->eap_widget);
+	gtk_widget_unparent (method->eap_widget);
 
 	if (method->size_group)
 		nma_eap_add_to_size_group (eap, method->size_group);
-	gtk_container_add (GTK_CONTAINER (vbox), eap_widget);
+	gtk_box_append (vbox, method->eap_widget);
 
 	nma_eap_unref (eap);
 
@@ -423,7 +421,7 @@ nma_eap_peap_new (NMAWs8021x *ws_8021x,
 	g_signal_connect (G_OBJECT (widget), "toggled",
 	                  (GCallback) nma_ws_changed_cb,
 	                  ws_8021x);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), ca_not_required);
+	gtk_check_button_set_active (GTK_CHECK_BUTTON (widget), ca_not_required);
 
 	widget = inner_auth_combo_init (method, connection, s_8021x, secrets_only);
 	inner_auth_combo_changed_cb (widget, (gpointer) method);
