@@ -2,7 +2,7 @@
 /*
  * Dan Williams <dcbw@redhat.com>
  *
- * Copyright 2007 - 2019 Red Hat, Inc.
+ * Copyright (C) 2007 - 2021 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -55,7 +55,7 @@ show_toggled_cb (GtkCheckButton *button, gpointer user_data)
 	NMAWsWpaPsk *self = NMA_WS_WPA_PSK (user_data);
 	gboolean visible;
 
-	visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+	visible = gtk_check_button_get_active (GTK_CHECK_BUTTON (button));
 	gtk_entry_set_visibility (GTK_ENTRY (self->wpa_psk_entry), visible);
 }
 
